"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DVCClient = void 0;
var Variable_1 = require("./Variable");
var Request_1 = require("./Request");
var Store_1 = __importDefault(require("./Store"));
var User_1 = require("./User");
var EventQueue_1 = require("./EventQueue");
var utils_1 = require("./utils");
var EventEmitter_1 = require("./EventEmitter");
var types_1 = require("@devcycle/types");
var RequestConsolidator_1 = require("./RequestConsolidator");
var logger_1 = require("./logger");
var DVCClient = /** @class */ (function () {
    function DVCClient(environmentKey, user, options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.user = user;
        this.options = options;
        this.environmentKey = environmentKey;
        this.variableDefaultMap = {};
        this.eventQueue = new EventQueue_1.EventQueue(environmentKey, this, options === null || options === void 0 ? void 0 : options.flushEventsMS);
        this.requestConsolidator = new RequestConsolidator_1.RequestConsolidator();
        this.eventEmitter = new EventEmitter_1.EventEmitter();
        this.logger = options.logger || (0, logger_1.dvcDefaultLogger)({ level: options.logLevel });
        var stubbedLocalStorage = {
            getItem: function () { return null; },
            setItem: function () { return undefined; },
            removeItem: function () { return undefined; },
            clear: function () { return undefined; },
            key: function () { return null; },
            length: 0
        };
        this.store = new Store_1.default(typeof window !== "undefined" ? window.localStorage : stubbedLocalStorage, this.logger);
        this.store.saveUser(this.user)
            .then(function () { return _this.logger.info('Successfully saved user to local storage!'); });
        this.onInitialized = (0, Request_1.getConfigJson)(environmentKey, this.user, this.logger, this.options)
            .then(function (config) {
            var _a;
            var oldConfig = _this.config;
            _this.config = config;
            if (checkIfEdgeEnabled(_this.config, _this.logger, (_a = _this.options) === null || _a === void 0 ? void 0 : _a.enableEdgeDB, true)) {
                if (!_this.user.isAnonymous) {
                    (0, Request_1.saveEntity)(_this.user, _this.environmentKey, _this.logger, _this.options)
                        .then(function (res) { return _this.logger.info("Saved response entity! ".concat(res)); });
                }
            }
            _this.store.saveConfig(config)
                .then(function () { return _this.logger.info('Successfully saved config to local storage'); });
            _this.eventEmitter.emitInitialized(true);
            _this.eventEmitter.emitFeatureUpdates((oldConfig === null || oldConfig === void 0 ? void 0 : oldConfig.features) || {}, _this.config.features);
            _this.eventEmitter.emitVariableUpdates((oldConfig === null || oldConfig === void 0 ? void 0 : oldConfig.variables) || {}, _this.config.variables, _this.variableDefaultMap);
            return _this;
        })
            .catch(function (err) {
            _this.eventEmitter.emitInitialized(false);
            _this.eventEmitter.emitError(err);
            return _this;
        });
    }
    DVCClient.prototype.onClientInitialized = function (onInitialized) {
        var _this = this;
        if (onInitialized && typeof onInitialized === 'function') {
            this.onInitialized
                .then(function () { return onInitialized(null, _this); })
                .catch(function (err) { return onInitialized(err); });
            return;
        }
        return this.onInitialized;
    };
    DVCClient.prototype.variable = function (key, defaultValue) {
        var _a;
        var _b, _c, _d, _e;
        if (defaultValue === undefined || defaultValue === null) {
            throw new Error('Default value is a required param');
        }
        var defaultValueKey = typeof defaultValue === 'string' ? defaultValue : JSON.stringify(defaultValue);
        var variable;
        if (this.variableDefaultMap[key] && this.variableDefaultMap[key][defaultValueKey]) {
            variable = this.variableDefaultMap[key][defaultValueKey];
        }
        else {
            var data = __assign({ key: key, defaultValue: defaultValue }, (_c = (_b = this.config) === null || _b === void 0 ? void 0 : _b.variables) === null || _c === void 0 ? void 0 : _c[key]);
            variable = new Variable_1.DVCVariable(data);
            this.variableDefaultMap[key] = __assign((_a = {}, _a[defaultValueKey] = variable, _a), this.variableDefaultMap[key]);
        }
        try {
            var variableFromConfig = (_e = (_d = this.config) === null || _d === void 0 ? void 0 : _d.variables) === null || _e === void 0 ? void 0 : _e[key];
            this.eventQueue.queueAggregateEvent({
                type: variable.isDefaulted
                    ? EventQueue_1.EventTypes.variableDefaulted
                    : EventQueue_1.EventTypes.variableEvaluated,
                target: variable.key,
                metaData: {
                    value: variable.value,
                    type: getTypeFromDefaultValue(variable.defaultValue, variable.key, this.logger),
                    _variable: variableFromConfig === null || variableFromConfig === void 0 ? void 0 : variableFromConfig._id
                }
            });
        }
        catch (e) {
            this.eventEmitter.emitError(e);
            this.logger.error("Error with queueing aggregate events ".concat(e));
        }
        return variable;
    };
    DVCClient.prototype.identifyUser = function (user, callback) {
        var _this = this;
        var promise = new Promise(function (resolve, reject) {
            try {
                _this.eventQueue.flushEvents();
                var updatedUser_1;
                if (user.user_id === _this.user.user_id) {
                    updatedUser_1 = _this.user.updateUser(user);
                }
                else {
                    updatedUser_1 = new User_1.DVCPopulatedUser(user, _this.options);
                }
                var oldConfig_1 = _this.config || {};
                _this.onInitialized.then(function () {
                    return _this.requestConsolidator.queue('identify', (0, Request_1.getConfigJson)(_this.environmentKey, updatedUser_1, _this.logger, _this.options));
                }).then(function (config) {
                    var _a;
                    _this.config = config;
                    if (checkIfEdgeEnabled(_this.config, _this.logger, (_a = _this.options) === null || _a === void 0 ? void 0 : _a.enableEdgeDB)) {
                        if (!updatedUser_1.isAnonymous) {
                            (0, Request_1.saveEntity)(updatedUser_1, _this.environmentKey, _this.logger, _this.options)
                                .then(function (res) { return _this.logger.info("Saved response entity! ".concat(res)); });
                        }
                    }
                    _this.store.saveConfig(config)
                        .then(function () { return _this.logger.info('Successfully saved config to local storage'); });
                    var oldFeatures = oldConfig_1.features || {};
                    var oldVariables = oldConfig_1.variables || {};
                    _this.eventEmitter.emitFeatureUpdates(oldFeatures, config.features);
                    _this.eventEmitter.emitVariableUpdates(oldVariables, config.variables, _this.variableDefaultMap);
                    return config.variables;
                }).then(function (variables) {
                    _this.user = updatedUser_1;
                    _this.store.saveUser(updatedUser_1)
                        .then(function () { return _this.logger.info('Successfully saved user to local storage!'); });
                    return resolve(variables || {});
                }).catch(function (err) { return Promise.reject(err); });
            }
            catch (err) {
                _this.eventEmitter.emitError(err);
                reject(err);
            }
        });
        if (callback && typeof callback == 'function') {
            promise.then(function (variables) { return callback(null, variables); })
                .catch(function (err) { return callback(err, null); });
            return;
        }
        return promise;
    };
    DVCClient.prototype.resetUser = function (callback) {
        var _this = this;
        var anonUser = new User_1.DVCPopulatedUser({ isAnonymous: true }, this.options);
        var promise = new Promise(function (resolve, reject) {
            try {
                _this.eventQueue.flushEvents();
                var oldConfig_2 = _this.config || {};
                _this.onInitialized.then(function () {
                    return _this.requestConsolidator.queue('identify', 
                    // don't send edgedb param for anonymous users
                    (0, Request_1.getConfigJson)(_this.environmentKey, anonUser, _this.logger, _this.options));
                }).then(function (config) {
                    _this.config = config;
                    _this.user = anonUser;
                    _this.store.saveConfig(config).then(function () {
                        _this.logger.info('Successfully saved config to local storage');
                    });
                    _this.store.saveUser(anonUser).then(function () {
                        _this.logger.info('Successfully saved user to local storage!');
                    });
                    var oldFeatures = oldConfig_2.features || {};
                    var oldVariables = oldConfig_2.variables || {};
                    _this.eventEmitter.emitFeatureUpdates(oldFeatures, config.features);
                    _this.eventEmitter.emitVariableUpdates(oldVariables, config.variables, _this.variableDefaultMap);
                    resolve(config.variables || {});
                }).catch(function (e) {
                    reject(e);
                });
            }
            catch (e) {
                _this.eventEmitter.emitError(e);
                reject(e);
            }
        });
        if (callback && typeof callback == 'function') {
            promise.then(function (variables) { return callback(null, variables); })
                .catch(function (err) { return callback(err, null); });
            return;
        }
        return promise;
    };
    DVCClient.prototype.allFeatures = function () {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.features) || {};
    };
    DVCClient.prototype.allVariables = function () {
        var _a;
        return ((_a = this.config) === null || _a === void 0 ? void 0 : _a.variables) || {};
    };
    DVCClient.prototype.subscribe = function (key, handler) {
        this.eventEmitter.subscribe(key, handler);
    };
    DVCClient.prototype.unsubscribe = function (key, handler) {
        this.eventEmitter.unsubscribe(key, handler);
    };
    DVCClient.prototype.track = function (event) {
        var _this = this;
        (0, utils_1.checkParamDefined)('type', event.type);
        this.onInitialized.then(function () {
            _this.eventQueue.queueEvent(event);
        });
    };
    DVCClient.prototype.flushEvents = function (callback) {
        return this.eventQueue.flushEvents().then(function () { return callback === null || callback === void 0 ? void 0 : callback(); });
    };
    return DVCClient;
}());
exports.DVCClient = DVCClient;
var checkIfEdgeEnabled = function (config, logger, enableEdgeDB, logWarning) {
    var _a, _b;
    if (logWarning === void 0) { logWarning = false; }
    if ((_b = (_a = config.project.settings) === null || _a === void 0 ? void 0 : _a.edgeDB) === null || _b === void 0 ? void 0 : _b.enabled) {
        return !!enableEdgeDB;
    }
    else {
        if (enableEdgeDB && logWarning) {
            logger.warn('EdgeDB is not enabled for this project. Only using local user data.');
        }
        return false;
    }
};
var getTypeFromDefaultValue = function (defaultValue, key, logger) {
    if (typeof defaultValue === 'boolean') {
        return types_1.VariableType.boolean;
    }
    else if (typeof defaultValue === 'number') {
        return types_1.VariableType.number;
    }
    else if (typeof defaultValue === 'string') {
        return types_1.VariableType.string;
    }
    else if (typeof defaultValue === 'object') {
        return types_1.VariableType.object;
    }
    else {
        logger.warn("The default value for variable ".concat(key, " is not of type Boolean, Number, String, or JSON"));
        return undefined;
    }
};
//# sourceMappingURL=Client.js.map